import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:pusher_websocket_flutter/pusher.dart';
import '../pay_unit_sdk.dart';
import 'blocs/PayUnitStream.dart';
import 'package:flutter/cupertino.dart';
import 'package:http_auth/http_auth.dart' as http_auth;
import 'package:native_progress_hud/native_progress_hud.dart';
import './Constant/Constant.dart'as constant;

class ApiService {
  Channel channel;
  final localNotificationsPlugin = FlutterLocalNotificationsPlugin();

  initializeNotifications() async {
    final initializeAndroid =
    AndroidInitializationSettings('@mipmap/ic_launcher');
    final initializeIOS = IOSInitializationSettings();
    final initSettings =
    InitializationSettings(android: initializeAndroid, iOS: initializeIOS);
    await localNotificationsPlugin.initialize(initSettings);
  }

  Future<void> singleNotification(String title, String body) async {
    final androidChannel = AndroidNotificationDetails(
      'channel-id',
      'PayUniy',
      'Scheduled notifications',
      importance: Importance.max,
      priority: Priority.max,
      color: Color(0xFF3AD9F2),
      visibility: NotificationVisibility.public,
      enableVibration: false,
      styleInformation: BigTextStyleInformation(''),
    );
    final iosChannel = IOSNotificationDetails();
    final platformChannel =
    NotificationDetails(android: androidChannel, iOS: iosChannel);
    await localNotificationsPlugin.show(3, title, body, platformChannel,
        payload: "");
  }

  Future<void> cancelNotification() async {
    return await localNotificationsPlugin.cancel(0);
  }

  Future<void> cancelAllNotifications() async {
    return await localNotificationsPlugin.cancelAll();
  }

  /// [context] : the actual context of the app
  /// [X_API_KEY] : the x-api-key
  /// [transactionAmount] : the amount you want to pay
  /// [transactionCallBackUrl] : your callBack url
  initiatePayment({
    @required context,
    @required String X_API_KEY,
    @required String productName,
    @required String transactionAmount,
    @required String transactionCallBackUrl,
    @required String description,
    @required merchandUserName,
    @required transaction_id,
    @required merchandPassword,
    @required sandbox,
    @required Function actionAfterProccess,
  }) async {
    try {

      progressHub("Please wait", context);
      await initPusher(transaction_id, actionAfterProccess, context);
      print(" is sandbox ? /initiatePayment: $sandbox");
      var data = jsonEncode({
        "description": description,
        "transaction_id": transaction_id,
        "total_amount": transactionAmount,
        "return_url": "https://eneocameroon.cm/index.php/en/"
      });
      var jsonResponse;
      var response = await getDigestAuth(merchandUserName, merchandPassword).post(
          "${sandbox == 'sandbox' ? constant.baseUrlSandBox : constant.baseUrl}/gateway/initialize",
          body: data,
          headers: getGlobalHeader(X_API_KEY));
      jsonResponse = json.decode(response.body);
      if (response.statusCode == 200 || response.statusCode == 201) {
        if (jsonResponse != null) {
          print(" 201 ou 200 response /initiate : ${response.body}");
          List result = await await getAllProviders(
            context: context,
            transactionId: jsonResponse['t_id'],
            transactionAmount: jsonResponse['t_sum'],
            transactionCallBackUrl: jsonResponse['t_url'],
            merchandPassword: merchandPassword,
            merchandUserName: merchandUserName,
            X_API_KEY: X_API_KEY,
            sandbox: sandbox,
            actionAfterProccess: actionAfterProccess,
          );
          return {"result": result, "transaction_id": transaction_id};
        }
      } else {
        print("Else bloc in initiatePayment : $jsonResponse");
        constant.makeToast(jsonResponse['message'], context, Colors.red);
        actionAfterProccess(transaction_id,
            "FAILED");
        return false;
      }
    } catch (e) {
      print("InitiatePayment catchError : $e");
      constant.makeToast("Something went wrong , please try again or later",
          context, Colors.red);
      actionAfterProccess(transaction_id,
          "FAILED");
      return false;
    } finally {
      closeDialog(context);
    }
  }

  /// [context] : the actual context of the app
  /// [transactionId] : the amount of the bill
  /// [transactionAmount] : the amount you want to pay
  /// [transactionCallBackUrl] : your callBack url
  getAllProviders({
    context,
    @required String transactionId,
    @required String transactionAmount,
    @required String transactionCallBackUrl,
    @required String merchandUserName,
    @required String merchandPassword,
    @required String X_API_KEY,
    @required String sandbox,
    @required Function actionAfterProccess,
  }) async {
    try {
      print(
          "This is the data of getAllProvider \ntransactionId :$transactionId \ntransactionAmount $transactionAmount  \ntransactionCallBackUrl $transactionCallBackUrl \nmerchandUserName $merchandUserName \merchandPassword $merchandPassword");
      print(" is sandbox ? /getAllProviders : $sandbox");
      var jsonResponse;
      var response = await getDigestAuth(merchandUserName, merchandPassword).get(
          "${sandbox == 'sandbox' ? constant.baseUrlSandBox : constant.baseUrl}/gateway/gateways?t_id=$transactionId&t_sum=$transactionAmount&t_url=$transactionCallBackUrl",
          headers: getGlobalHeader(X_API_KEY));
      jsonResponse = json.decode(response.body);
      print("This is the result of getAllProvider : ${response.body}");
      if (response.statusCode == 200) {
        if (jsonResponse != null) {
          return json.decode(response.body)['data'];
        }
      } else {
        if (jsonResponse != null) {
          print(" Else response /providers : $jsonResponse");
          closeDialog(context);
          actionAfterProccess(transactionId,
              "FAILED");
          return [];
        }
      }
    } catch (e) {
      print(" catch error /providers : ${e}");
      closeDialog(context);
      constant.makeToast("Something went wrong , please try again or later",
          context, Colors.red);
      actionAfterProccess(transactionId,
          "FAILED");
    } finally {
      closeDialog(context);
    }
  }

  /// [context] : the actual context of the app
  /// [transactionId] : the amount of the bill
  /// [transactionAmount] : the amount you want to pay
  /// [transactionCallBackUrl] : your callBack url
  getPaymentStatus({
    @required context,
    @required String transactionId,
    @required String paymentRef,
    @required String payToken,
    @required String merchandUserName,
    @required String merchandPassword,
    @required String provider_short_tag,
    @required Function actionAfterProccess,
    @required String X_API_KEY,
    @required String sandbox,
  }) async {
    try {
      print(" is sandbox ? /getPaymentStatus: $sandbox");
      print("I am waiting for push notification ");
      // callback;
      const time = const Duration(seconds: 4);
      Timer _timer;
      _timer = new Timer.periodic(time, (Timer timer) async {
        var jsonResponse;
        var response = await getDigestAuth(merchandUserName, merchandPassword).get(
            "${sandbox == 'sandbox' ? constant.baseUrlSandBox : constant.baseUrl}/gateway/paymentstatus/$provider_short_tag/$transactionId?payment_ref=${paymentRef}&pay_token=${payToken}",
            headers: getGlobalHeader(X_API_KEY));
        if (response.statusCode == 200) {
          _timer.cancel();
        } else {
          if (jsonResponse != null) {
            print(" Else response /getPaymentStatus : $jsonResponse");
            constant.makeToast(jsonResponse['error'], context, Colors.red);

            closeDialog(context);
            _timer.cancel();
          }
        }
      });
    } catch (e) {
      print(" catch error /providers : $e");
      // closeDialog(context);
      constant.makeToast("Something went wrong , please try again or later",
          context, Colors.red);
    } finally {
      // closeDialog(context);
    }
  }

  /// [context] : the actual context of the app
  /// [transactionId] : the amount of the bill
  /// [transactionAmount] : the amount you want to pay
  /// [transactionCallBackUrl] : your callBack url
  /// [provider_short_tag] : the tag of the provider
  /// [phoneNumber] : the phone number of the client
  /// [userName] : the user name of the client
  makePayment({
    @required context,
    @required String X_API_KEY,
    @required String transactionAmount,
    @required String transactionCallBackUrl,
    @required String provider_short_tag,
    @required String phoneNumber,
    @required Function actionAfterProccess,
    @required String userName,
    @required merchandUserName,
    @required transaction_id,
    @required merchandPassword,
    @required sandbox,
    @required currency,
    @required paymentType,
  }) async {

    var response;
    var data = currency != null
        ? jsonEncode({
      'gateway': provider_short_tag,
      'amount': transactionAmount,
      'transaction_id': transaction_id,
      'return_url': transactionCallBackUrl,
      'phone_number': phoneNumber,
      'description': "Pay unit flutter sdk",
      "name": provider_short_tag,
      "currency": currency,
      "paymentType": paymentType
    })
        : jsonEncode({
      'gateway': provider_short_tag,
      'amount': transactionAmount,
      'transaction_id': transaction_id,
      'return_url': transactionCallBackUrl,
      'phone_number': phoneNumber,
      'description': "Pay unit flutter sdk",
      "name": provider_short_tag,
      "paymentType": paymentType
    });
    try {
      progressHub("Please wait", context);
      var jsonResponse;
      if (provider_short_tag == 'mtnmomo' || provider_short_tag == 'orange') {
        constant.closeAndToastPaymentAreMake(context);
        response = await getDigestAuth(merchandUserName, merchandPassword).post(
            "${sandbox == 'sandbox' ? constant.baseUrlSandBox : constant.baseUrl}/gateway/makepayment",
            body: data,
            headers: getGlobalHeader(X_API_KEY));
      } else if (provider_short_tag == 'eu') {
        constant.closeAndToastPaymentAreMake(context);
        response = await getDigestAuth(merchandUserName, merchandPassword).post(
            "${sandbox == 'sandbox' ? constant.baseUrlSandBox : constant.baseUrl}/gateway/makepayment",
            body: data,
            headers: getGlobalHeader(X_API_KEY));
      } else if (provider_short_tag == 'stripe') {
        response = await getDigestAuth(merchandUserName, merchandPassword).post(
            "${sandbox == 'sandbox' ? constant.baseUrlSandBox : constant.baseUrl}/gateway/makepayment",
            body: data,
            headers: getGlobalHeader(X_API_KEY));
      } else if (provider_short_tag == 'paypal') {
        response = await getDigestAuth(merchandUserName, merchandPassword).post(
            "${sandbox == 'sandbox' ? constant.baseUrlSandBox : constant.baseUrl}/gateway/makepayment",
            body: data,
            headers: getGlobalHeader(X_API_KEY));
      }
      if (response.statusCode == 200 || response.statusCode == 201) {
        constant.makeToast(
            json.decode(response.body)['message'], context, Colors.green);
        jsonResponse = json.decode(response.body)["data"];
        if(jsonResponse["access_token"]==null){

        }
        if (jsonResponse["access_token"]!=null) {
          var obj = {
            "order_id": jsonResponse["order_id"],
            "amount": jsonResponse["amount"],
            "currency": jsonResponse["currency"],
            "client_id": jsonResponse["client_id"],
            "accessToken": jsonResponse["access_token"],
          };
          closeDialog(context);
          getPaymentStatus(
              transactionId: transaction_id,
              paymentRef: jsonResponse["payment_ref"],
              payToken: jsonResponse["pay_token"],
              merchandUserName: merchandUserName,
              merchandPassword: merchandPassword,
              provider_short_tag: provider_short_tag,
              actionAfterProccess: actionAfterProccess,
              context: context,
              sandbox: sandbox,
              X_API_KEY: X_API_KEY);
          return obj;
        }else{
          var obj = {
            "order_id": jsonResponse["order_id"],
            "amount": jsonResponse["amount"],
            "currency": jsonResponse["currency"],
            "client_id": jsonResponse["client_id"],
          };
          closeDialog(context);
          getPaymentStatus(
              transactionId: transaction_id,
              paymentRef: jsonResponse["payment_ref"],
              payToken: jsonResponse["pay_token"],
              merchandUserName: merchandUserName,
              merchandPassword: merchandPassword,
              provider_short_tag: provider_short_tag,
              actionAfterProccess: actionAfterProccess,
              context: context,
              sandbox: sandbox,
              X_API_KEY: X_API_KEY);
          return obj;
        }

      } else {
        var responseData = json.decode(response.body);
        print(" Else response /makePayment : $responseData");
        actionAfterProccess(transaction_id,
            "FAILED");
        if (responseData != null) {
          payUnitStream.paymentSink.addError("Error");
          closeDialog(context);
          print("This is the error : $responseData");
          constant.makeToast(responseData['error'], context, Colors.red);
        }
      }
    } catch (e) {
      actionAfterProccess(transaction_id,
          "FAILED");
      print("Make payment error : $e");
      closeDialog(context);
      constant.makeToast("Something went wrong , please try again or later",
          context, Colors.red);
    } finally {
      // closeDialog(context);
    }
  }

  Future<void> initPusher(
      channelName, actionAfterProccess, BuildContext context) async {
    print("Transaction id pusher : $channelName");

    try {
      await Pusher.init('8e6ec079f9d817bd73eb', PusherOptions(cluster: 'ap4'));
    } catch (e) {
      print(e.message);
    }
    //connect to pusher
    Pusher.connect(
        onConnectionStateChange: (ConnectionStateChange connectionState) async {
          print("Pusher connection state : ${connectionState.currentState}");
        }, onError: (ConnectionError e) {
      print("Error: ${e.message}");
    });

    //connect to channel
    channel = await Pusher.subscribe(channelName);
    //get data
    channel.bind('payment-event', (last) {
      print("this is pusher : ${last.data}");

      if (jsonDecode(last.data)['transaction_status'] == "SUCCESS") {
        singleNotification('PayUnit',
            'Transaction :${jsonDecode(last.data)['transaction_id']} , is success');
        payUnitStream.paymentSink.add(false);
        print(
            "Pusher transaction :${jsonDecode(last.data)['transaction_status']}");

        closeDialog(context);
        constant.makeToast(
            "Your transaction : ${jsonDecode(last.data)['transaction_id']} have the status: ${jsonDecode(last.data)['transaction_status']}",
            context,
            jsonDecode(last.data)['transaction_status'] == "SUCCESS"
                ? Colors.green
                : Colors.red);
        actionAfterProccess(jsonDecode(last.data)['transaction_id'],
            jsonDecode(last.data)['transaction_status']);
        closeDialog(context);
      } else {
        closeDialog(context);
        singleNotification('PayUnit',
            'Transaction No :${jsonDecode(last.data)['transaction_id']} , is failed');
        constant.makeToast(
            "Your transaction failed , please try again or later",
            context,
            Colors.red);
        payUnitStream.paymentSink.addError("error");
      }
    });
  }

  void unbindEvent() {
    channel.unbind('payment-event');
  }

  void unSubscribePusher(String channelName) {
    Pusher.unsubscribe(channelName);
  }

  getDigestAuth(String username, String password) {
    var digest = http_auth.DigestAuthClient(username, password);
    return digest;
  }

  getGlobalHeader(String X_API_KEY) {
    return {
      'Content-Type': 'application/json',
      'X-API-KEY': X_API_KEY,
    };
  }

  progressHub(msg, context) {
    NativeProgressHud.showWaitingWithText(msg);
  }

  closeDialog(context) {
    NativeProgressHud.hideWaiting();
  }

}
